@@ -16,8 +16,5 @@
 #   along with ebpub.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-# The key used in request.session to represent the user ID.
-USER_SESSION_KEY = 'user_id'
-
 # The key used in request.session to represent the user's e-mail.
 EMAIL_SESSION_KEY = 'email'
