@@ -16,12 +16,12 @@
 #   along with ebpub.  If not, see <http://www.gnu.org/licenses/>.
 #
 
-from django.contrib.auth.models import User
-import constants # relative import
+from django.contrib import auth
+from django.contrib.auth.models import User as BaseUser
 
 UNDEFINED = 123
 
-class LazyUser(User):
+class LazyUser(BaseUser):
     # This class is a transparent wrapper around User that hits the database
     # only after an attribute is accessed.
     def __init__(self, user_id):
@@ -42,13 +42,18 @@ class LazyUser(User):
                 self._user_cache = None
         return getattr(self._user_cache, name)
 
+    def get_profile(self):
+        # For some reason this doesn't automatically work.
+        func = self.__getattr__('get_profile')
+        return func()
+
 class LazyUserDescriptor(object):
     # This class uses a Python descriptor so that a LazyUser isn't
     # actually created until request.user is accessed.
     def __get__(self, request, obj_type=None):
         if not hasattr(request, '_cached_user'):
             try:
-                user_id = request.session[constants.USER_SESSION_KEY]
+                user_id = request.session[auth.SESSION_KEY]
             except KeyError:
                 from ebpub.accounts.models import AnonymousUser
                 user = AnonymousUser()
