@@ -59,25 +59,32 @@ def email_for_subscription(alert, start_date, frequency):
     start_datetime = datetime.datetime(start_date.year, start_date.month, start_date.day)
     yesterday = datetime.date.today() - datetime.timedelta(days=1)
     end_datetime = datetime.datetime.combine(yesterday, datetime.time(23, 59, 59, 9999)) # the end of yesterday
-    # Order by schema__id to group schemas together.
-    qs = NewsItem.objects.select_related().filter(schema__is_public=True)
+
+    from ebpub.utils.view_utils import get_schema_manager_for_user
+    manager = get_schema_manager_for_user(alert.user)
+    allowed_schemas = manager.allowed_schema_ids()
+    qs = NewsItem.objects.select_related().filter(schema__id__in=allowed_schemas)
     if alert.include_new_schemas:
+        # We saved an opt-out list.
         if alert.schemas:
             qs = qs.exclude(schema__id__in=alert.schemas.split(','))
     else:
+        # We saved an opt-in list.
         if alert.schemas:
             qs = qs.filter(schema__id__in=alert.schemas.split(','))
 
-    if alert.block:
-        place_name, place_url = alert.block.pretty_name, alert.block.url()
-        place = alert.block
-        search_buffer = make_search_buffer(alert.block.location.centroid, alert.radius)
+    if alert.block_center:
+        place = alert._get_block()
+        place_name, place_url = place.pretty_name, place.url()
+        search_buffer = make_search_buffer(place.geom.centroid, alert.radius)
         qs = qs.filter(location__bboverlaps=search_buffer)
+
     elif alert.location:
         place_name, place_url = alert.location.name, alert.location.url()
         place = alert.location
         qs = qs.filter(newsitemlocation__location__id=alert.location.id)
 
+    # Order by schema__id to group schemas together.
     news_qs = qs.filter(schema__is_event=False,
                         pub_date__range=(start_datetime, end_datetime),
                         ).order_by('-schema__importance', 'schema__id', '-item_date', '-id')
