@@ -19,8 +19,11 @@
 # These might end up in settings eventually, but for now it's fine to
 # hardcode them.
 
-BLOCK_RADIUS_CHOICES = {'1': .0015, '3': .0035, '8': .007} # number of blocks -> number of geographic degrees
+BLOCK_RADIUS_CHOICES = {'1': .0015, '3': .0035, '8': .007} # number of blocks -> number of geographic degrees ... should this vary with latitude? oh well.
 BLOCK_RADIUS_COOKIE_NAME = 'radius'
 BLOCK_RADIUS_DEFAULT = '8'
 HIDE_ADS_COOKIE_NAME = 'h'
-HIDE_SCHEMA_INTRO_COOKIE_NAME = 'schemaintro'
+
+# How far from a block center various model geometries can be,
+# and still be considered on that block.
+BLOCK_FUZZY_DISTANCE_METERS = 15
