@@ -20,7 +20,7 @@
 Unit tests for db.schemafilters.
 """
 
-from client import RequestFactory
+from ebpub.utils.testing import RequestFactory
 from django.core import urlresolvers
 from django.test import TestCase
 from ebpub.db.schemafilters import FilterChain
@@ -28,10 +28,10 @@ from ebpub.db.schemafilters import FilterError
 from ebpub.db.urlresolvers import filter_reverse
 from ebpub.db.views import BadAddressException
 from ebpub.db import models
+import datetime
 import mock
 import random
 
-
 class TestNewsitemFilter(TestCase):
 
     def test_getitem(self):
@@ -42,6 +42,41 @@ class TestNewsitemFilter(TestCase):
         self.assertRaises(KeyError, fil.__getitem__, 'bar')
 
 
+class TestIdFilter(TestCase):
+
+    fixtures = ('test-schemafilter-views.json',)
+
+    def test_validate(self):
+        from ebpub.db.schemafilters import IdFilter
+        fil = IdFilter(request=None, context={}, queryset=None)
+        self.assertEqual(fil.validate(),
+                         {'filter_key': 'id',
+                          'option_list': [],
+                          'param_label': 'id',
+                          'param_name': 'id',
+                          'select_multiple': True}
+                         )
+
+    def test_apply__no_ids(self):
+        from ebpub.db.schemafilters import IdFilter
+        fil = IdFilter(request=None, context={}, queryset=None, ids=[])
+        self.assertEqual(list(fil.apply()), [])
+        from ebpub.db.models import NewsItem
+        self.assert_(NewsItem.objects.all().count() >= 1,
+                     "we should have some items")
+
+    def test_apply__some_ids(self):
+        from ebpub.db.schemafilters import IdFilter
+        from ebpub.db.models import NewsItem
+        items = NewsItem.objects.all()[:2]
+        self.assert_(len(items), "we should have some items")
+        expected_ids = [item.id for item in items]
+        fil = IdFilter(request=None, context={}, queryset=None,
+                       ids=expected_ids)
+        got_ids = [item.id for item in fil.apply()]
+        self.assertEqual(sorted(expected_ids), sorted(got_ids))
+
+
 class TestSchemaFilter(TestCase):
 
     fixtures = ('test-schemafilter-views.json',)
@@ -62,6 +97,28 @@ class TestSchemaFilter(TestCase):
         for item in fil.qs:
             self.assertEqual(item.schema.natural_key(), schema.natural_key())
 
+    def test_apply__multi_schema(self):
+        from ebpub.db.schemafilters import SchemaFilter
+        schema = models.Schema.objects.get(slug='crime')
+        fil = SchemaFilter(request=None, context={}, queryset=None,
+                           schema=[schema, schema])
+        fil.apply()
+        self.assert_(len(fil.qs) > 0)
+        for item in fil.qs:
+            self.assertEqual(item.schema.natural_key(), schema.natural_key())
+
+
+    @mock.patch('ebpub.db.schemafilters.get_schema_manager')
+    def test_apply__with_request(self, mock_get_mgr):
+        from ebpub.db.schemafilters import SchemaFilter
+        schema = models.Schema.objects.get(slug='crime')
+        request = mock.Mock()
+        mock_get_mgr.return_value = models.Schema.objects
+        fil = SchemaFilter(request=request, context={}, queryset=None,
+                           schema=[schema, schema])
+        fil.apply()
+        self.assertEqual(mock_get_mgr.call_count, 1)
+
 
 class TestLocationFilter(TestCase):
 
@@ -80,15 +137,18 @@ class TestLocationFilter(TestCase):
         filt = LocationFilter(req, context, None, *reverse_args[1:])
         return filt
 
-    def test_filter__errors(self):
+    def test_filter__empty(self):
         self.assertRaises(FilterError, self._make_filter)
 
-    def test_filter_by_location_choices(self):
+    def test_filter_by_location__choices(self):
         filt = self._make_filter('neighborhoods')
         more_needed = filt.validate()
-        self.assertEqual(more_needed['lookup_type'], u'Neighborhood')
-        self.assertEqual(more_needed['lookup_type_slug'], 'neighborhoods')
-        self.assert_(len(more_needed['lookup_list']) > 0)
+        self.assertEqual(more_needed['filter_key'], u'location')
+        self.assertEqual(more_needed['param_name'], 'locations')
+        self.assertEqual(more_needed['param_label'], u'Neighborhood')
+        self.assert_(len(more_needed['option_list']) > 0)
+        self.assertEqual(filt.get_query_params(),
+                         {'locations': 'neighborhoods'})
 
     @mock.patch('ebpub.db.schemafilters.models.Location.objects.filter')
     def test_filter_by_location__no_locations(self, mock_location_query):
@@ -109,7 +169,12 @@ class TestLocationFilter(TestCase):
         filt.apply()
         expected_loc = models.Location.objects.get(slug='hood-1')
         self.assertEqual(filt.location_object, expected_loc)
-        # TODO: have some NewsItems overlapping this location?
+        self.assertEqual(filt.get_query_params(),
+                         {'locations': 'neighborhoods,hood-1'})
+        # Items 1 & 2 should match this query because there's a corresponding
+        # NewsItemLocation.
+        self.assertEqual(sorted([item.id for item in filt.qs]),
+                         [1, 2])
 
 
 class TestBlockFilter(TestCase):
@@ -178,7 +243,6 @@ class TestDateFilter(TestCase):
         self.assertRaises(FilterError, self._make_filter, 'by-date')
         self.assertRaises(FilterError, self._make_filter, 'by-date', 'bogus')
         self.assertRaises(FilterError, self._make_filter, 'by-date', 'bogus', 'bogus')
-        self.assertRaises(FilterError, self._make_filter, 'by-date', '2011-04-07')
 
     def test_filter__ok(self):
         filt = self._make_filter('by-date', '2006-11-08', '2006-11-09')
@@ -187,6 +251,11 @@ class TestDateFilter(TestCase):
         self.assertEqual(filt.value, u'Nov. 8, 2006 - Nov. 9, 2006')
         self.assertEqual(self.mock_qs.filter.call_args, ((), filt.kwargs))
 
+    def test__single_arg_date(self):
+        filt = self._make_filter('by-date', '2011-04-07')
+        filt.apply()
+        self.assertEqual(filt.value, u'April 7, 2011')
+
 
     def test_filter__ok__one_day(self):
         filt = self._make_filter('by-date', '2006-11-08', '2006-11-08')
@@ -196,7 +265,7 @@ class TestDateFilter(TestCase):
         self.assertEqual(self.mock_qs.filter.call_args, ((), filt.kwargs))
 
     def test_pub_date_filter(self):
-        filt = self._make_filter('by-pub-date', '2006-11-08', '2006-11-09')
+        filt = self._make_filter('by-pubdate', '2006-11-08', '2006-11-09')
         from ebpub.db.schemafilters import PubDateFilter
         filt2 = PubDateFilter(filt.request, filt.context, filt.qs,
                               '2006-11-08', '2006-11-09')
@@ -207,6 +276,57 @@ class TestDateFilter(TestCase):
         self.assertEqual(self.mock_qs.filter.call_args, ((), filt2.kwargs))
 
 
+class TestAttributeFilter(TestCase):
+
+    def _mock_schemafield(self, name='mock-sf', pretty_name='Mock SF', **kwargs):
+        # This is because the 'name' argument to mock.Mock
+        # means something special, so we have to assign to it
+        # after instantiation.
+        sf = mock.Mock(pretty_name=pretty_name, **kwargs)
+        sf.name = name
+        return sf
+
+    def test_init__no_args(self):
+        from ebpub.db.schemafilters import AttributeFilter
+        schemafield = self._mock_schemafield(name='mock-sf')
+        qs = {}
+        request = context = None
+        filter = AttributeFilter(request, context, qs, schemafield=schemafield)
+        self.assertEqual(filter.get_query_params(),
+                         {'by-mock-sf': ''})
+
+
+    def test_init__with_date(self):
+        from ebpub.db.schemafilters import AttributeFilter
+        schemafield = self._mock_schemafield(name='mock-sf')
+        when = datetime.date(2009, 1, 23)
+        qs = {}
+        request = context = None
+        filter = AttributeFilter(request, context, qs, when, schemafield=schemafield)
+        self.assertEqual(filter.get_query_params(),
+                         {'by-mock-sf': '2009-01-23'})
+
+    def test_init__with_datetime(self):
+        from ebpub.db.schemafilters import AttributeFilter
+        schemafield = self._mock_schemafield(name='mock-sf')
+        when = datetime.datetime(2009, 1, 23, 22, 40)
+        qs = {}
+        request = context = None
+        filter = AttributeFilter(request, context, qs, when, schemafield=schemafield)
+        self.assertEqual(filter.get_query_params(),
+                         {'by-mock-sf': '2009-01-23T22:40:00'})
+
+    def test_init__with_time(self):
+        from ebpub.db.schemafilters import AttributeFilter
+        schemafield = self._mock_schemafield(name='mock-sf')
+        when = datetime.time(22, 40)
+        qs = {}
+        request = context = None
+        filter = AttributeFilter(request, context, qs, when, schemafield=schemafield)
+        self.assertEqual(filter.get_query_params(), 
+                         {'by-mock-sf': '22:40:00'})
+
+
 class TestBoolFilter(TestCase):
 
     fixtures = ('crimes.json',)
@@ -240,7 +360,8 @@ class TestBoolFilter(TestCase):
     def test_filter__more_needed(self):
         filt = self._make_filter('by-arrests')
         more_needed = filt.validate()
-        self.assertEqual(more_needed['lookup_type_slug'], 'arrests')
+        self.assertEqual(more_needed['filter_key'], 'arrests')
+        self.assertEqual(more_needed['param_name'], 'by-arrests')
 
 
 class TestLookupFilter(TestCase):
@@ -253,10 +374,11 @@ class TestLookupFilter(TestCase):
         url = filter_reverse('crime', [url_args])
         req = RequestFactory().get(url)
         context = {'schema': crime}
-        sf_name = url_args[0][3:]   # 'by-foo' -> 'foo'
+        sf_name, lookups = url_args[0], url_args[1:]
+        sf_name = sf_name[3:]   # 'by-foo' -> 'foo'
         sf = models.SchemaField.objects.get(name=sf_name)
         self.mock_qs = mock.Mock()
-        filt = LookupFilter(req, context, self.mock_qs, *url_args[1:], schemafield=sf)
+        filt = LookupFilter(req, context, self.mock_qs, *lookups, schemafield=sf)
         return filt
 
     def test_filter__errors(self):
@@ -270,23 +392,39 @@ class TestLookupFilter(TestCase):
         filt = self._make_filter('by-beat')
         more_needed = filt.validate()
         self.assert_(more_needed)
-        self.assertEqual(more_needed['lookup_type'], 'Beat')
-        self.assertEqual(more_needed['lookup_type_slug'], 'beat')
-        self.assertEqual(len(more_needed['lookup_list']), 2)
+        self.assertEqual(more_needed['filter_key'], 'beat')
+        self.assertEqual(more_needed['param_name'], 'by-beat')
+        self.assertEqual(more_needed['param_label'], 'Beats')
+        self.assert_(len(more_needed['option_list']) > 0)
 
-    def test_filter__ok(self):
-        filt = self._make_filter('by-beat', 'beat-214', )
+    def test_filter__ok_single(self):
         filt = self._make_filter('by-beat', 'beat-214', )
         self.assertEqual(filt.validate(), {})
         filt.apply()
-        self.assertEqual(filt.look.id, 214)
+        self.assertEqual(filt.lookups[0].id, 214)
         self.assertEqual(self.mock_qs.by_attribute.call_args,
-                         ((filt.schemafield, filt.look), {'is_lookup': True}))
+                         ((filt.schemafield, filt.lookups), {'is_lookup': True}))
         self.assertEqual(filt.value, 'Police Beat 214')
         self.assertEqual(filt.short_value, 'Police Beat 214')
         self.assertEqual(filt.label, 'Beat')
         self.assertEqual(filt.argname, 'by-beat')
 
+    def test_filter__ok_multi(self):
+        filt = self._make_filter('by-beat', 'beat-214', 'beat-64')
+        self.assertEqual(filt.validate(), {})
+        filt.apply()
+        self.assertEqual(filt.lookups[0].id, 214)
+        self.assertEqual(filt.lookups[1].id, 64)
+        self.assertEqual(self.mock_qs.by_attribute.call_args,
+                         ((filt.schemafield, filt.lookups), {'is_lookup': True}))
+        self.assertEqual(filt.value, 'Police Beat 214, Police Beat 64')
+        self.assertEqual(filt.short_value, 'Police Beat 214, Police Beat 64')
+        self.assertEqual(filt.label, 'Beat')
+        self.assertEqual(filt.argname, 'by-beat')
+        self.assertEqual(filt.get_query_params(),
+                         {'by-beat': 'beat-214,beat-64'})
+
+
 class TestTextFilter(TestCase):
 
     fixtures = ('crimes.json',)
@@ -436,20 +574,75 @@ class TestFilterChain(TestCase):
         self.assertEqual(chain['date'].start_date, datetime.date(2011, 8, 1))
         self.assertEqual(chain['date'].end_date, datetime.date(2011, 8, 31))
 
+    def test_add__pubdate(self):
+        # Key gets overridden.
+        chain = FilterChain()
+        import datetime
+        chain.add('pubdate', datetime.date(2011, 8, 13))
+        self.assert_(chain.has_key('date'))
+        self.failIf(chain.has_key('pubdate'))
+        self.assertEqual(chain['date'].start_date, datetime.date(2011, 8, 13))
 
+    def test_add__bogus_keyword_arg(self):
+        chain = FilterChain()
+        self.assertRaises(TypeError, chain.add, 'date', '2011-01-1', foo='bar')
+
+    def test_add__no_value(self):
+        chain = FilterChain()
+        self.assertRaises(FilterError, chain.add, 'date')
+
+    def test_update_from_request__empty(self):
+        request = mock.Mock()
+        class StubQueryDict(dict):
+            def getlist(self, key):
+                return []
+            def copy(self):
+                return StubQueryDict(self.items())
+
+        request.GET = StubQueryDict()
+        chain = FilterChain(request=request)
+        chain.update_from_request({})
+        self.assertEqual(len(chain), 0)
+
+    def test_add_by_place_id__bad(self):
+        chain = FilterChain()
+        from django.http import Http404
+        self.assertRaises(Http404, chain.add_by_place_id, '')
+        self.assertRaises(Http404, chain.add_by_place_id, 'blah')
+        self.assertRaises(Http404, chain.add_by_place_id, 'b:123.1')
+        self.assertRaises(Http404, chain.add_by_place_id, 'l:9999')
+
+    @mock.patch('ebpub.utils.view_utils.get_object_or_404')
+    def test_add_by_place_id(self, mock_get_object_or_404):
+        chain = FilterChain()
+        from ebpub.streets.models import Block
+        from ebpub.db.schemafilters import BlockFilter
+        block = Block(city='city', street_slug='street_slug',
+                      pretty_name='pretty_name',
+                      street_pretty_name='street_pretty_name',
+                      street='street',
+                      from_num='123', to_num='456',
+                      )
+        mock_get_object_or_404.return_value = block
+        chain.add_by_place_id('b:123.1')
+        self.assert_(isinstance(chain['location'], BlockFilter))
+
+    def test_add__id(self):
+        from ebpub.db.schemafilters import IdFilter
+        chain = FilterChain()
+        chain.add('id', [1, 2, 3])
+        self.assert_(isinstance(chain['id'], IdFilter))
 
 
 class TestUrlNormalization(TestCase):
 
     fixtures = ('test-schemafilter-views.json',)
 
-
     def setUp(self):
         super(TestUrlNormalization, self).setUp()
         self._patcher1 = mock.patch('ebpub.streets.models.proper_city')
         self.proper_city = self._patcher1.start()
         self.proper_city.return_value = 'chicago'
-
         self._patcher2 = mock.patch('ebpub.db.schemafilters.SmartGeocoder.geocode')
         self.mock_geocode = self._patcher2.start()
 
@@ -460,11 +653,10 @@ class TestUrlNormalization(TestCase):
 
     def _make_chain(self, url):
         request = RequestFactory().get(url)
-        argstring = request.path.split('filter/', 1)[-1]
         crime = models.Schema.objects.get(slug='crime')
         context = {'schema': crime}
         chain = FilterChain(request=request, context=context, schema=crime)
-        chain.update_from_request(argstring=argstring, filter_sf_dict={})
+        chain.update_from_request(filter_sf_dict={})
         return chain
 
     def test_urls__ok(self):
@@ -475,7 +667,7 @@ class TestUrlNormalization(TestCase):
 
     def test_urls__intersection(self):
         url = urlresolvers.reverse(
-            'ebpub-schema-filter', args=['crime', 'filter']) + '?address=foo+and+bar'
+            'ebpub-schema-filter', args=['crime']) + '?address=foo+and+bar'
         from ebpub.streets.models import Block, Intersection, BlockIntersection
         intersection = mock.Mock(spec=Intersection)
         blockintersection = mock.Mock(spec=BlockIntersection)
@@ -490,7 +682,7 @@ class TestUrlNormalization(TestCase):
 
     def test_urls__bad_intersection(self):
         url = urlresolvers.reverse(
-            'ebpub-schema-filter', args=['crime', 'filter']) + '?address=foo+and+bar'
+            'ebpub-schema-filter', args=['crime']) + '?address=foo+and+bar'
         from ebpub.streets.models import Block, Intersection, BlockIntersection
         intersection = mock.Mock(spec=Intersection)
         blockintersection = mock.Mock(spec=BlockIntersection)
@@ -508,7 +700,7 @@ class TestUrlNormalization(TestCase):
 
 
     def test_make_url__bad_address(self):
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         url += '?address=anything'  # doesn't matter because of mock_geocode
 
         from ebpub.geocoder.parser.parsing import ParsingError
@@ -537,7 +729,7 @@ class TestUrlNormalization(TestCase):
             'block': block,
             }
 
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         url += '?address=216+n+wabash+st&radius=8' # geocode result is mocked anyway.
 
         expected_url = filter_reverse('crime', [('streets', 'wabash-ave', '216-299n-s', '8-blocks'),])
@@ -546,7 +738,7 @@ class TestUrlNormalization(TestCase):
 
     def test_make_url__bad_dates(self):
         from ebpub.db.views import BadDateException
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         # Error if either date is way too old.
         self.assertRaises(BadDateException, self._make_chain,
                           url + '?start_date=12/31/1899&end_date=01/01/2011')
@@ -561,42 +753,36 @@ class TestUrlNormalization(TestCase):
 
 
     def test_make_url__date_query(self):
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         url += '?start_date=2010/12/01&end_date=2011/01/01'
         chain = self._make_chain(url)
-        expected = filter_reverse('crime', [('by-date', '2010-12-01', '2011-01-01')])
+        expected = filter_reverse('crime', [('start_date', '2010-12-01'),
+                                            ('end_date', '2011-01-01')])
         self.assertEqual(chain.make_url(), expected)
 
     def test_make_url__no_such_schemafield(self):
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         url += '?textsearch=foo&q=hello+goodbye'
         self.assertRaises(models.SchemaField.DoesNotExist,
                           self._make_chain, url)
 
     def test_make_url__textsearch_query(self):
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         url += '?textsearch=status&q=hello+goodbye'
         chain = self._make_chain(url)
         expected = filter_reverse('crime', [('by-status', 'hello goodbye')])
         self.assertEqual(chain.make_url(), expected)
 
-    def test_make_url__both_args_and_query(self):
-        url = filter_reverse('crime', [('by-date', '2011-04-05', '2011-04-06')])
-        url += '?textsearch=status&q=bar'
-        chain = self._make_chain(url)
-        expected = filter_reverse('crime', [('by-date', '2011-04-05', '2011-04-06'),
-                                            ('by-status', 'bar')])
-        self.assertEqual(chain.make_url(), expected)
-
-
     def test_make_url__preserves_other_query_params_sorted(self):
-        url = filter_reverse('crime', [('by-date', '2011-04-05', '2011-04-06')])
-        url += '?textsearch=status&q=bar'
-        # Add some params that we don't know about.
-        url += '&B=no&A=yes'
+        url = filter_reverse('crime', [('start_date', '2011-04-05'),
+                                       ('end_date', '2011-04-06')])
+        url += '&textsearch=status&q=bar'
+        # Add the extra params.
+        url += '&zzz=yes&A=no'
         chain = self._make_chain(url)
-        expected = filter_reverse('crime', [('by-date', '2011-04-05', '2011-04-06'),
+        expected = filter_reverse('crime', [('start_date', '2011-04-05'),
+                                            ('end_date', '2011-04-06'),
                                             ('by-status', 'bar')])
-        expected += '?A=yes&B=no'
+        # The extra params should end up sorted alphanumerically.
+        expected = expected.replace('?', '?A=no&') + '&zzz=yes'
         self.assertEqual(chain.make_url(), expected)
-
