@@ -121,6 +121,7 @@ class LocationDetailTestCase(BaseTestCase):
         self.assertEqual(response.status_code, 200)
         self.assertEqual(len(response.context['newsitem_list']), 1)
         self.assertEqual(response.context['newsitem_list'][0].title, 'crime title 1')
+        # We don't get 'crime title 2' in this query because it's in the future.
 
     def test_location_overview(self):
         url = urlresolvers.reverse('ebpub-location-overview',
@@ -262,17 +263,13 @@ class TestSchemaFilterView(BaseTestCase):
         self.assertEqual(response.status_code, 302)
         self.assertEqual(response['location'], 'http://testserver/whee/')
 
-    def test_filter__charting_disallowed_redirect(self):
-        # could make a different fixture, but, meh.
-        crime = models.Schema.objects.get(slug='crime')
-        crime.allow_charting = False
-        crime.save()
+    def test_filter__bad_params(self):
         url = filter_reverse('crime', [('by-foo', 'bar')])
+        url = url.replace(urllib.quote('='), 'X')
         response = self.client.get(url)
-        self.assertEqual(response.status_code, 301)
-        self.assertEqual(response['location'], 'http://testserver/crime/')
+        self.assertEqual(response.status_code, 404)
 
-    @mock.patch('ebpub.db.schemafilters.FilterChain.update_from_query_params')
+    @mock.patch('ebpub.db.schemafilters.FilterChain.update_from_request')
     def test_filter__bad_date(self, mock_update):
         from ebpub.db.views import BadDateException
         mock_update.side_effect = BadDateException("oh no")
@@ -280,14 +277,9 @@ class TestSchemaFilterView(BaseTestCase):
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
 
-    def test_filter__bad_params(self):
-        url = filter_reverse('crime', [('by-foo', 'bar')])
-        url = url.replace(urllib.quote('='), 'X')
-        response = self.client.get(url)
-        self.assertEqual(response.status_code, 404)
-
     def test_filter_by_daterange(self):
-        url = filter_reverse('crime', [('by-date', '2006-11-01', '2006-11-30')])
+        url = filter_reverse('crime', [('start_date', '2006-11-01'),
+                                       ('end_date', '2006-11-30')])
         response = self.client.get(url)
         self.assertContains(response, 'Clear')
         self.assertNotContains(response, "crime title 1")
@@ -295,7 +287,8 @@ class TestSchemaFilterView(BaseTestCase):
         self.assertContains(response, "crime title 3")
 
     def test_filter_by_pubdate_daterange(self):
-        url = filter_reverse('crime', [('by-pub-date', '2006-11-01', '2006-11-30')])
+        url = filter_reverse('crime', [('start_pubdate', '2006-11-01'),
+                                       ('end_pubdate', '2006-11-30')])
         response = self.client.get(url)
         self.assertContains(response, 'Clear')
         self.assertNotContains(response, "crime title 1")
@@ -305,31 +298,57 @@ class TestSchemaFilterView(BaseTestCase):
     def test_filter__only_one_date_allowed(self):
         url = filter_reverse('crime',
                              [('by-date', '2006-11-01', '2006-11-30'),
-                              ('by-pub-date', '2006-11-01', '2006-11-30')])
+                              ('by-pubdate', '2006-11-01', '2006-11-30')])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
 
 
+    def test_filter__date_missing(self):
+        url = filter_reverse('crime', [('start_date', '')])
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 302)
+        self.assert_(response['location'].endswith('/crime/'))
+
     def test_filter__invalid_daterange(self):
-        url = filter_reverse('crime', [('by-date', '')])
+        url = filter_reverse('crime', [('start_date', 'whoops'),
+                                       ('end_date', 'ouchie')])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
-        url = filter_reverse('crime', [('by-date', 'whoops', 'ouchie')])
-        response = self.client.get(url)
-        self.assertEqual(response.status_code, 404)
-        url = filter_reverse('crime', [('by-date', '2006-11-30', 'ouchie')])
+        url = filter_reverse('crime', [('start_date', '2006-11-30'),
+                                       ('end_date', 'ouchie')])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
 
 
     def test_filter_by_day(self):
-        url = filter_reverse('crime', [('by-date', '2006-09-26', '2006-09-26')])
+        url = filter_reverse('crime', [('start_date', '2006-09-26'),
+                                       ('end_date', '2006-09-26')])
         response = self.client.get(url)
         self.assertContains(response, "crime title 1")
         self.assertNotContains(response, "crime title 2")
         self.assertNotContains(response, "crime title 3")
 
 
+    def test_filter__by_date__legacy_redirects(self):
+        base = 'http://testserver' + filter_reverse('crime', [])
+        expected_to_actual = (
+            (filter_reverse('crime', [('by-date', '2011-09-25', '2012-10-31')]),
+             base + '?end_date=2012-10-31&start_date=2011-09-25'
+             ),
+            (filter_reverse('crime', [('by-pubdate', '2011-09-25', '2012-10-31')]),
+             base + '?end_pubdate=2012-10-31&start_pubdate=2011-09-25'
+             ),
+
+            )
+        for expected, actual in expected_to_actual:
+            response = self.client.get(expected)
+            self.assertEqual(response.status_code, 302)
+            self.assertEqual(response['location'], actual)
+            response2 = self.client.get(actual)
+            self.assertEqual(response2.status_code, 200)
+
+
+
     def test_filter_by_attributes__text(self):
         url = filter_reverse('crime', [('by-status', 'status 9-19')])
         response = self.client.get(url)
@@ -383,7 +402,7 @@ class TestSchemaFilterView(BaseTestCase):
         self.assertEqual(response.status_code, 302)
         fixed_url = filter_reverse('crime', [
                 ('streets', 'wabash-ave', '216-299n-s', '8-blocks')])
-        self.assert_(response['location'].endswith(urllib.unquote(fixed_url)))
+        self.assertEqual(response['location'], 'http://testserver' + fixed_url)
 
     @mock.patch('ebpub.streets.models.proper_city')
     def test_filter_by_block(self, mock_proper_city):
@@ -440,9 +459,14 @@ class TestSchemaFilterView(BaseTestCase):
         self.assertEqual(response.template[0].name, 'db/filter_lookup_list.html')
 
     def test_filter__by_m2m_lookup_attr(self):
-        # XXX todo.
-        # I don't think there are any m2m lookups in crimes.json yet
-        pass
+        url = filter_reverse('crime', [('by-tag', 'tag-1', 'tag-2',),])
+        response = self.client.get(url)
+        self.assertEqual(response.status_code, 200)
+        self.assertContains(response, 'Tag 1')
+        self.assertContains(response, 'Tag 2')
+        self.assertNotContains(response, 'Tag 999')
+        self.assertContains(response, 'crime title 1')
+        self.assertContains(response, 'crime title 2')
 
     def test_filter__by_boolean_attr__true(self):
         url = filter_reverse('crime', [('by-arrests', 'yes',),])
@@ -469,10 +493,10 @@ class TestSchemaFilterView(BaseTestCase):
         self.assertEqual(response.status_code, 404)
 
     def test_filter__invalid_argname(self):
+        # These are ignored.
         url = filter_reverse('crime', [('bogus-key', 'bogus-value')])
         response = self.client.get(url)
-        self.assertEqual(response.status_code, 404)
-
+        self.assertEqual(response.status_code, 200)
 
     @mock.patch('ebpub.db.schemafilters.logger')
     @mock.patch('ebpub.db.models.AggregateFieldLookup.objects.filter')
@@ -483,8 +507,8 @@ class TestSchemaFilterView(BaseTestCase):
         mock_item = mock.Mock()
         mock_item.return_value = mock_item
         mock_item.alters_data = False
-        mock_aggr().select_related().order_by.return_value = [mock_item] * 100
-        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime', 'filter'])
+        mock_aggr().select_related().order_by.return_value = [mock_item] * 999
+        url = urlresolvers.reverse('ebpub-schema-filter', args=['crime'])
         response = self.client.get(url)
         self.assertEqual(response.status_code, 200)
         lookup_list = response.context['lookup_list']
@@ -494,20 +518,20 @@ class TestSchemaFilterView(BaseTestCase):
 
     def test_filter__pagination__invalid_page(self):
         url = filter_reverse('crime', [('by-status', 'status 9-19')])
-        url += '?page=oops'
+        url += '&page=oops'
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
 
     def test_filter__pagination__empty_page(self):
         url = filter_reverse('crime', [('by-status', 'status 9-19')])
-        url += '?page=99'
+        url += '&page=99'
         response = self.client.get(url)
         self.assertEqual(response.status_code, 404)
 
     @mock.patch('ebpub.db.views.FilterChain', spec=True)
     def test_filter__pagination__has_more(self, mock_chain):
         url = filter_reverse('crime', [('by-status', 'status 9-19')])
-        url += '?page=2'
+        url += '&page=2'
         # We can mock the FilterChain to get a very long list of NewsItems
         # without actually creating them in the db, but it means
         # also mocking a ton of methods used by schema_filter or filter.html.
@@ -517,7 +541,7 @@ class TestSchemaFilterView(BaseTestCase):
         mock_qs = mock.Mock()
         mock_qs.filter.return_value = mock_qs
         newsitem = models.NewsItem.objects.all()[0]
-        mock_qs.order_by.return_value = [newsitem] * 100
+        mock_qs.order_by.return_value = [newsitem] * 999
 
         mock_chain.return_value = mock_chain
         mock_chain.apply.return_value = mock_qs
