@@ -16,9 +16,16 @@
 #   along with ebpub.  If not, see <http://www.gnu.org/licenses/>.
 #
 
+"""
+Flexible geocoding against our models, including db.Location,
+streets.Place, streets.Block, streets.Intersection ...
+"""
+
+from django.conf import settings
 from django.core.exceptions import ObjectDoesNotExist
 from django.db.models import Q
 from ebpub.geocoder.parser.parsing import normalize, parse, ParsingError
+from ebpub.utils.text import address_to_block
 import logging
 import re
 
@@ -52,7 +59,7 @@ class InvalidBlockButValidStreet(GeocodingException):
         self.block_number = block_number
         self.street_name = street_name
         self.block_list = block_list
-    
+
 class Address(dict):
     "A simple container class for representing a single street address."
     def __init__(self, *args, **kwargs):
@@ -71,6 +78,13 @@ class Address(dict):
             return self["point"].x
     lng = longitude
 
+    @property
+    def location(self):
+        """Everything else full_geocode() can return has a .location,
+        so this is here for consistency of API."""
+        if self["point"]:
+            return self["point"]
+
     def __unicode__(self):
         return u", ".join([self[k] for k in ["address", "city", "state", "zip"]])
 
@@ -79,6 +93,7 @@ class Address(dict):
         """
         Builds an Address object from a GeocoderCache result object.
         """
+        #  This should probably use the normal Django cache, see ticket #163
         fields = {
             'address': cached.address,
             'city': cached.city,
@@ -166,15 +181,23 @@ class Geocoder(object):
         logger.debug('geocoded: %r to %s' % (location, result))
         return result
 
+
 class AddressGeocoder(Geocoder):
+    """
+    Treats the location_string as an address and looks for a matching Block.
+    """
     def _do_geocode(self, location_string):
         # Parse the address.
         try:
             locations = parse(location_string)
         except ParsingError, e:
-            raise
+            raise e
 
         all_results = []
+
+        # For capturing streets with matching name but no matching block.
+        invalid_block_args = []
+
         for loc in locations:
             logger.debug('AddressGeocoder: Trying %r' % loc)
             loc_results = self._db_lookup(loc)
@@ -187,6 +210,8 @@ class AddressGeocoder(Geocoder):
                              % loc['street'])
                 try:
                     misspelling = StreetMisspelling.objects.get(incorrect=loc['street'])
+                    # TODO: stash away the original 'street' value for
+                    # possible disambiguation later? ticket #295
                     loc['street'] = misspelling.correct
                     logger.debug(' ... corrected to %r' % loc['street'])
                 except StreetMisspelling.DoesNotExist:
@@ -201,7 +226,7 @@ class AddressGeocoder(Geocoder):
                                  % loc['suffix'])
                     loc_results = self._db_lookup(dict(loc, suffix=None))
                 # Next, try looking for the street, in case the street
-                # exists but the address doesn't.
+                # (without any suffix) exists but the address doesn't.
                 if not loc_results and loc['number']:
                     kwargs = {'street': loc['street']}
                     sided_filters = []
@@ -212,9 +237,14 @@ class AddressGeocoder(Geocoder):
                     from ebpub.streets.models import Block
                     b_list = Block.objects.filter(*sided_filters, **kwargs).order_by('predir', 'from_num', 'to_num')
                     if b_list:
+                        # We got some blocks with the bare street name.
+                        # Might be InvalidBlockButValidStreet, but we don't
+                        # want to raise that till we've tried all locations,
+                        # in case there's a better one coming up.
                         logger.debug("Street %r exists but block %r doesn't"
                                      % (b_list[0].street_pretty_name, loc['number']))
-                        raise InvalidBlockButValidStreet(loc['number'], b_list[0].street_pretty_name, b_list)
+                        invalid_block_args = [loc['number'], b_list[0].street_pretty_name, b_list]
+
             if loc_results:
                 logger.debug(u'Success. Adding to results: %s' % [unicode(r) for r in loc_results])
                 all_results.extend(loc_results)
@@ -222,12 +252,16 @@ class AddressGeocoder(Geocoder):
                 logger.debug('... Got nothing.')
 
         if not all_results:
-            raise DoesNotExist("Geocoder db couldn't find this location: %r" % location_string)
+            if invalid_block_args:
+                raise InvalidBlockButValidStreet(*invalid_block_args)
+            else:
+                raise DoesNotExist("Geocoder db couldn't find this location: %r" % location_string)
         elif len(all_results) == 1:
             return all_results[0]
         else:
             raise AmbiguousResult(all_results)
 
+
     def _db_lookup(self, location):
         """
         Given a location dict as returned by parse(), looks up the address in
@@ -245,6 +279,7 @@ class AddressGeocoder(Geocoder):
                 street=location['street'],
                 number=location['number'],
                 predir=location['pre_dir'],
+                prefix=location['prefix'],
                 suffix=location['suffix'],
                 postdir=location['post_dir'],
                 city=location['city'],
@@ -256,6 +291,7 @@ class AddressGeocoder(Geocoder):
             raise
         return [self._build_result(location, block, geocoded_pt) for block, geocoded_pt in blocks]
 
+
     def _build_result(self, location, block, geocoded_pt):
         return Address({
             'address': unicode(" ".join([str(s) for s in [location['number'], block.predir, block.street_pretty_name, block.postdir] if s])),
@@ -269,7 +305,12 @@ class AddressGeocoder(Geocoder):
             'wkt': str(block.location),
         })
 
+
 class BlockGeocoder(AddressGeocoder):
+    """
+    Geocodes the location_string as a streets.Block.
+    """
+
     def _do_geocode(self, location_string):
         m = block_re.search(location_string)
         if not m:
@@ -277,7 +318,12 @@ class BlockGeocoder(AddressGeocoder):
         new_location_string = ' '.join(m.groups())
         return AddressGeocoder._do_geocode(self, new_location_string)
 
+
 class IntersectionGeocoder(Geocoder):
+    """
+    Geocodes the location_string as a streets.Intersection.
+    """
+
     def _do_geocode(self, location_string):
         sides = intersection_re.split(location_string)
         if len(sides) != 2:
@@ -308,6 +354,7 @@ class IntersectionGeocoder(Geocoder):
         else:
             raise AmbiguousResult(list(all_results), "Intersections DB returned %s results" % len(all_results))
 
+
     def _db_lookup(self, street_a, street_b):
         # Avoid circular import.
         from ebpub.streets.models import Intersection
@@ -326,6 +373,7 @@ class IntersectionGeocoder(Geocoder):
             raise DoesNotExist("Intersection db query failed: %r" % e)
         return [self._build_result(i) for i in intersections]
 
+
     def _build_result(self, intersection):
         return Address({
             'address': intersection.pretty_name,
@@ -353,7 +401,12 @@ class IntersectionGeocoder(Geocoder):
 #             raise DoesNotExist("Segment query failed: %r" % e)
 #         # TODO: Make a line from the two points, and return that.
 
+
 class SmartGeocoder(Geocoder):
+    """
+    Checks whether the location_string looks like an Intersection, Block,
+    or Address, and delegates to the appropriate Geocoder subclass.
+    """
     def _do_geocode(self, location_string):
         if intersection_re.search(location_string):
             logger.debug('%r looks like an intersection' % location_string)
@@ -365,3 +418,180 @@ class SmartGeocoder(Geocoder):
             logger.debug('%r assumed to be an address' % location_string)
             geocoder = AddressGeocoder()
         return geocoder._do_geocode(location_string)
+
+
+def full_geocode(query, search_places=True, convert_to_block=True, guess=False,
+                 **disambiguation_kwargs):
+    """
+    Tries the full geocoding stack on the given query (a string):
+
+    * Normalizes whitespace/capitalization
+    * Searches the Misspelling table to corrects location misspellings
+    * Searches the Location table
+    * Failing that, searches the Place table (if search_places is True)
+    * Failing that, uses the SmartGeocoder to parse this as an address, block,
+      or intersection
+    * Failing that, raises whichever error is raised by the geocoder --
+      except AmbiguousResult, in which case all possible results are
+      returned
+
+    Returns a dictionary of {type, result, ambiguous}, where ambiguous is True
+    or False, and type can be on of these strings:
+
+    * 'location' -- in which case result is a Location object.
+    * 'place' -- in which case result is a Place object. (This is only
+      possible if search_places is True.)
+    * 'address' -- in which case result is an Address object as returned
+      by geocoder.geocode().
+    * 'block' -- in which case result is an Address object based on the block.
+
+    When ``ambiguous`` is True in the output dict, ``result`` will be
+    a list of objects, and vice versa.
+
+    You can control behavior with ambiguous results in several ways:
+
+    * By passing guess=True, only the first result will be returned.
+
+    * By passing additional kwargs such as ``zipcode``, ``city``, or
+      ``state``, they will be used to attempt to disambiguate address
+      or block results as needed.  Keys should be keys in each Address result;
+      invalid keys have no effect.
+
+    * By default, *if* the exact address is not matched, it will be
+      rounded down to the nearest 100, eg.  '123 Main St' will be
+      converted to '100 block of Main St', and tried again with
+      BlockGeocoder.  This is enabled by default; you can pass
+      ``convert_to_block=False`` to turn it off.
+
+    """
+    # Local import to avoid circular imports.
+    from ebpub.db.models import Location, LocationSynonym
+    from ebpub.streets.models import Place, PlaceSynonym
+
+    # Search the Location table.
+    try:
+        canonical_loc = LocationSynonym.objects.get_canonical(query)
+        loc = Location.objects.get(normalized_name=canonical_loc)
+    except Location.DoesNotExist:
+        pass
+    else:
+        logger.debug('geocoded %r to Location %s' % (query, loc))
+        return {'type': 'location', 'result': loc, 'ambiguous': False}
+
+    # Search the Place table, for stuff like "Sears Tower".
+    if search_places:
+        canonical_place = PlaceSynonym.objects.get_canonical(query)
+        places = Place.objects.filter(normalized_name=canonical_place)
+        if len(places) == 1:
+            logger.debug(u'geocoded %r to Place %s' % (query, places[0]))
+            return {'type': 'place', 'result': places[0], 'ambiguous': False}
+        elif len(places) > 1:
+            # TODO: Places don't know about city, state, zip...
+            # so we can't disambiguate.
+            logger.debug(u'geocoded %r to multiple Places: %s' % (query, unicode(places)))
+            return {'type': 'place', 'result': places, 'ambiguous': True}
+
+    # Try geocoding this as an address.
+    geocoder = SmartGeocoder(use_cache=getattr(settings, 'EBPUB_CACHE_GEOCODER', False))
+    try:
+        result = geocoder.geocode(query)
+    except AmbiguousResult, e:
+        logger.debug('Multiple addresses for %r' % query)
+        # The disambiguation args (zipcode, city, state,...)
+        # are not included in the initial pass because it
+        # is often too picky, yielding no results when there is a
+        # legitimate nearby zipcode or city identified in either the address
+        # or street number data.
+        results = disambiguate(e.choices, guess=guess, **disambiguation_kwargs)
+        if not results:
+            logger.debug("Disambiguate returned nothing, should not happen")
+            results = e.choices
+        if len(results) > 1:
+            return {'type': 'address', 'result': results, 'ambiguous': True}
+        else:
+            return {'type': 'address', 'result': results[0], 'ambiguous': False}
+
+    except InvalidBlockButValidStreet, e:
+        result = {
+            'type': 'block',
+            'ambiguous': True,
+            'result': e.block_list,
+            'street_name': e.street_name,
+            'block_number': e.block_number,
+            }
+        if convert_to_block:
+            # If the exact address couldn't be geocoded, try using the
+            # normalized block name.
+            block_name = address_to_block(query)
+            if block_name != query:
+                try:
+                    result['result'] = BlockGeocoder()._do_geocode(block_name)
+                    result['result']['address'] = block_name
+                    result['ambiguous'] = False
+                    logger.debug('Resolved %r to block %r' % (query, block_name))
+                except (InvalidBlockButValidStreet, AmbiguousResult):
+                    pass
+        if result['ambiguous']:
+            logger.debug('Invalid block for %r, returning all possible blocks' % query)
+        return result
+
+    except:
+        raise
+
+    logger.debug('SmartGeocoder for %r returned %s' % (query, result))
+    return {'type': 'address', 'result': result, 'ambiguous': False}
+
+
+def disambiguate(geocoder_results, guess=False, **kwargs):
+    """Disambiguate a list of geocoder results based on city, state, zip.
+    Result will be a list, which may be the original list or a subset of it.
+
+    If guess==True, returns the first remaining result, which could be
+    wildly off from what you expect (eg. in the case of 'invalid block
+    but valid street')... so use with caution.
+    """
+    if not kwargs:
+        if guess and geocoder_results:
+            logger.debug("Nothing to disambiguate on, returning first result.")
+            return [geocoder_results[0]]
+        else:
+            logger.debug("Nothing to disambiguate or guess, returning all.")
+            return geocoder_results
+
+    filtered_results = geocoder_results[:]
+
+    for key, target_val in kwargs.items():
+        # special case: allow passing either form
+        if key == 'zipcode':
+            key = 'zip'
+
+        if not target_val:
+            continue
+
+        target_val = target_val.lower()
+        previous_results = filtered_results[:]
+        filtered_results = [r for r in previous_results
+                            if r.get(key, '').lower() == target_val]
+        if not filtered_results:
+            # Oops, too restrictive. Ignore this disambiguator and
+            # try again with any remaining  disambiguators.
+            logger.debug(
+                "Ambiguous results, but none are in %s %r" % (key, target_val))
+            filtered_results = previous_results
+            continue
+
+        if len(filtered_results) > 1:
+            # Still too many.
+            logger.debug(
+                "Still ambiguous results in %s %r..." % (key, target_val))
+            continue
+
+        elif len(filtered_results) == 1:
+            # Yay, successfully disambiguated.
+            return filtered_results
+
+    if len(filtered_results) > 1 and guess:
+        logger.debug("Still ambiguous results, guessing first.")
+        return [filtered_results[0]]
+
+    return filtered_results
