@@ -3,8 +3,8 @@
 /***************************************************/
 
 function _obapi(path) {
-    var OB_API_ROOT = '/api/dev1';
-    return OB_API_ROOT + path;
+    /* you are expected to set OB_API_ROOT in your template */
+    return OB_API_ROOT.replace(/\/\b/, "") + "/" + path.replace(/\b\//, "");
 }
 
 function _report_error(error) {
@@ -65,7 +65,7 @@ OBMapItemList.prototype._setupNewLayer = function(layer) {
         'loadend': this.update,
         'scope': this
     });
-}
+};
 
 OBMapItemList.prototype.update = function() {
     this._findVisibleItems();
@@ -77,14 +77,20 @@ OBMapItemList.prototype._resetPager = function() {
     this.page = 0;
     this.pages = Math.ceil(this.items.length / this.listLength);
     if (this.pages > 1) {
-        var pagerHTML = '<a class="nav-prev" href="#">&larr;prev</a>&nbsp;';
-        pagerHTML +=    '<span class="page-number">' + (this.page + 1) + '</span>&nbsp;of&nbsp;';
-        pagerHTML += this.pages + '&nbsp;<a class="nav-next" href="#">next&rarr;</a></div>';
+        // "Page 50 of 70 first | <prev | next> | last"
+        var pagerHTML = 'Page&nbsp;<span class="page-number">' + (this.page+1);
+        pagerHTML +=  '</span>&nbsp;of&nbsp;<span class="page-count">'+ this.pages + '</span>';
+        pagerHTML += '&nbsp;<a class="nav-first" href="#">first</a>&nbsp;|';
+        pagerHTML += '&nbsp;<a class="nav-prev" href="#">&larr;prev</a>&nbsp;|';
+        pagerHTML += '&nbsp;<a class="nav-next" href="#">next&rarr;</a>&nbsp;|';
+        pagerHTML += '&nbsp;<a class="nav-last" href="#">last</a>';
 
         this.el.find('.map-item-list-pager').html(pagerHTML);
         var prev = this.el.find("a.nav-prev");
         var next = this.el.find("a.nav-next");
-        
+        var first = this.el.find("a.nav-first");
+        var last = this.el.find("a.nav-last");
+
         // Clicking next or previous replaces the nav links html.
         var thisList = this;
         prev.click(function(e) {
@@ -95,6 +101,15 @@ OBMapItemList.prototype._resetPager = function() {
             e.preventDefault();
             thisList.nextPage();
         });
+        first.click(function(e) {
+            e.preventDefault();
+            thisList.jumpToPage(0);
+        });
+        last.click(function(e) {
+            e.preventDefault();
+            thisList.jumpToPage(thisList.pages - 1);
+        });
+
     }
     else {
         this.el.find('.map-item-list-pager').html("");
@@ -102,13 +117,17 @@ OBMapItemList.prototype._resetPager = function() {
 };
 
 OBMapItemList.prototype.nextPage = function() {
-    this.page = (this.page + 1) % this.pages;
-    this.el.find('.page-number').html(this.page + 1); 
-    this._refreshPage();
+    var page = (this.page + 1) % this.pages;
+    this.jumpToPage(page);
 };
 
 OBMapItemList.prototype.prevPage = function() {
-    this.page = (this.page - 1 + this.pages) % this.pages; 
+    var page = (this.page - 1 + this.pages) % this.pages;
+    this.jumpToPage(page);
+};
+
+OBMapItemList.prototype.jumpToPage = function(page) {
+    this.page = page;
     this.el.find('.page-number').html(this.page + 1);
     this._refreshPage();
 };
@@ -135,19 +154,8 @@ OBMapItemList.prototype._findVisibleItems = function() {
         }
     }
     this.items.sort(function(a,b) {
-        var at = a.attributes.openblock_type; 
-        var bt = b.attributes.openblock_type; 
-        
-        if (at != bt) {
-            /* if types do not match, order by type */
-            ak = at;
-            bk = bt;
-        }
-        else {
-            ak = a.attributes.sort || a.attributes.name || a.attributes.title;
-            bk = b.attributes.sort || a.attributes.name || a.attributes.title;
-        }
-        
+        ak = a.attributes.sort || a.attributes.name || a.attributes.title || a.attributes.openblock_type;
+        bk = b.attributes.sort || b.attributes.name || b.attributes.title || b.attributes.openblock_type;
         return ((ak < bk) ? -1 : ((bk < ak) ? 1 : 0));
     });
 };
@@ -155,7 +163,7 @@ OBMapItemList.prototype._findVisibleItems = function() {
 OBMapItemList.prototype._refreshPage = function() {
     /* gather items on page */
     var items = [];
-    var start = this.page*this.listLength
+    var start = this.page * this.listLength;
     for (var i = 0; i < this.listLength; i++) {
         var cur = start + i; 
         if (cur < this.items.length) {
@@ -265,7 +273,8 @@ var OpenblockMergeBBOX = OpenLayers.Class(OpenLayers.Strategy.BBOX, {
 
         var existingFeatureIds = {};
         var newFeatures = [];
-        for (var i = 0; i < this.layer.features.length; i++) {
+        var i = 0;
+        for (i = 0; i < this.layer.features.length; i++) {
             for (var j = 0; j < this.layer.features[i].cluster.length; j++) {
                 var feature = this.layer.features[i].cluster[j];
                 existingFeatureIds[feature.attributes.id] = true;
@@ -279,9 +288,10 @@ var OpenblockMergeBBOX = OpenLayers.Class(OpenLayers.Strategy.BBOX, {
         if(features && features.length > 0) {
             var remote = this.layer.projection;
             var local = this.layer.map.getProjectionObject();
+            var len=features.length;
             if(!local.equals(remote)) {
                 var geom;
-                for(var i=0, len=features.length; i<len; ++i) {
+                for(i=0; i<len; ++i) {
                     geom = features[i].geometry;
                     if(geom) {
                         geom.transform(remote, local);
@@ -289,7 +299,7 @@ var OpenblockMergeBBOX = OpenLayers.Class(OpenLayers.Strategy.BBOX, {
                 }
             }
 
-            for (var i = 0; i < features.length; i++) {
+            for (i = 0; i < len; i++) {
                 var feature = features[i];
                 if (existingFeatureIds[feature.attributes.id] != true) {
                     newFeatures.push(feature);
@@ -460,13 +470,13 @@ var OpenblockPermalink = OpenLayers.Class(OpenLayers.Control.Permalink, {
     },
 
     createParams: function(center, zoom, layers) {
-        var layers = '';
+        var layerIds = '';
         for (var i = 0; i < this.map.layers.length; i++) {
             var layer = this.map.layers[i];
             if (layer.getVisibility() == true && typeof(layer.layerConfig) != "undefined") {
                 var layerid = layer.layerConfig.id;
                 if (typeof(layerid) != "undefined") {
-                    layers += layerid;
+                    layerIds += layerid;
                 }
             }
         }
@@ -475,7 +485,7 @@ var OpenblockPermalink = OpenLayers.Class(OpenLayers.Control.Permalink, {
         var params = {
             'c': this._encodeLonLat(this.map.center),
             'z': this.map.getZoom(),
-            'l': layers
+            'l': layerIds
         };
         extra_params = this.obmap.options.permalink_params;
         if (typeof(extra_params) != 'undefined') {
@@ -561,7 +571,6 @@ var OBMap = function(options) {
     this._configurePopup();
     this._configureLayers();
 
-    
 };
 
 OBMap.prototype.events = null;
@@ -574,7 +583,11 @@ OBMap.prototype.map_options = {
     units: "m",
     numZoomLevels: 19,
     maxResolution: 156543.03390625,
-    maxExtent: new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34)
+    maxExtent: new OpenLayers.Bounds(-20037508.34, -20037508.34, 20037508.34, 20037508.34),
+    controls: [
+        new OpenLayers.Control.Navigation(),
+        new OpenLayers.Control.ZoomPanel()
+    ]
 };
 
 OBMap.prototype._initBasicMap = function() {
@@ -609,13 +622,14 @@ OBMap.prototype._initBasicMap = function() {
 };
 
 OBMap.prototype._configureLayers = function() {
+    var i = 0;
     if (typeof(this.options.locations) != 'undefined') {
-        for (var i = 0; i < this.options.locations.length; i++) {
+        for (i = 0; i < this.options.locations.length; i++) {
             this.loadLocationBorder(this.options.locations[i]);
         }
     }
     if (typeof(this.options.layers) != 'undefined') {
-        for (var i = 0; i < this.options.layers.length; i++) {
+        for (i = 0; i < this.options.layers.length; i++) {
             this.loadFeatureLayer(this.options.layers[i]);
         }
     }
@@ -791,7 +805,7 @@ OBMap.prototype.loadAllNewsLayers = function() {
     /* load news item types, create a layer for each */
 
     jQuery.ajax({
-        url: _obapi('/items/types.json'),
+        url: _obapi('items/types.json'),
         dataType: 'json',
         context: this,
         success: function(data, status, request) {
@@ -800,7 +814,7 @@ OBMap.prototype.loadAllNewsLayers = function() {
                     var itemType = data[slug];
                     var layerConfig = {
                         title:  itemType["plural_name"],
-                        url:    _obapi('/items.json'),
+                        url:    _obapi('items.json'),
                         params: {'type': slug, limit: 150},
                         bbox: true,
                         visible: true
@@ -818,7 +832,7 @@ OBMap.prototype.loadAllNewsLayers = function() {
 OBMap.prototype.loadAllPlaceLayers = function() {
     /* load place types, create a layer for each */
     jQuery.ajax({
-        url: _obapi('/places/types.json'),
+        url: _obapi('places/types.json'),
         dataType: 'json',
         context: this,
         success: function(data, status, request) {
@@ -827,7 +841,7 @@ OBMap.prototype.loadAllPlaceLayers = function() {
                     var itemType = data[slug];
                     var layerConfig = {
                         title: itemType["plural_name"],
-                        url: _obapi('/places/' + slug +'.json'),
+                        url: _obapi('places/' + slug +'.json'),
                         params: {limit: 150},
                         bbox: true,
                         visible: false
@@ -837,7 +851,7 @@ OBMap.prototype.loadAllPlaceLayers = function() {
             }
         },
         error: function(request, status, error) {
-            _report_error('An error occurred loading news item types.');
+            _report_error('An error occurred loading place layers.');
         }
     });
 };
