@@ -19,8 +19,16 @@
 from django.db import models
 from django.db.models.signals import post_save
 from ebpub.db.models import Schema
-import ebpub.accounts.models
 from ebpub.openblockapi.apikey.models import MAX_KEYS
+import ebpub.accounts.models
+import warnings
+
+with warnings.catch_warnings():
+    # Okay, silencing deprecation warnings is not great,
+    # but a lot of OpenBlock's intended audience seem to think
+    # that DeprecationWarnings are actually errors.
+    warnings.simplefilter("ignore")
+    from jsonfield.fields import JSONField
 
 class HiddenSchema(models.Model):
     user_id = models.IntegerField()
@@ -42,15 +50,18 @@ class HiddenSchema(models.Model):
 
 class Profile(models.Model):
     """
-    Currently just metadata about the user's API keys, but this would
-    be the logical place to hang preferences information and such;
-    hence putting this in the preferences app.
+    User account metadata: API key limits, plus a big bag of JSON
+    for arbitrary other stuff.
     """
     user = models.ForeignKey(ebpub.accounts.models.User, unique=True)
 
     max_keys = models.IntegerField(default=MAX_KEYS,
                                    help_text="How many API keys can this user have?")
 
+    properties = JSONField(null=True, blank=True, default=dict,
+                           help_text="A bag of JSON for holding arbitrary user metadata."
+                           )
+
     def available_keys(self):
         """
         How many *more* API keys can this user get?
