@@ -27,7 +27,6 @@ from django import forms, template
 from django.http import HttpResponse
 from django.shortcuts import render_to_response
 from django.utils.encoding import force_unicode
-from ebpub.utils.logutils import log_exception
 from ebpub.streets.models import Block, Street, BlockIntersection, \
     Intersection, Suburb, Place, PlaceType, PlaceSynonym, StreetMisspelling
 from ebpub.geocoder import SmartGeocoder, AmbiguousResult, GeocodingException
@@ -127,7 +126,7 @@ class PlaceAdmin(OSMModelAdmin):
              url(r'^export/csv$', self.admin_site.admin_view(self.export_csv_view), name="streets_place_export_csv")
          )
          return my_urls + urls
-    
+
     def export_csv_view(self, request):
 
         if request.method == 'GET':
@@ -278,7 +277,7 @@ class PlaceAdmin(OSMModelAdmin):
                     message = 'Updated place %s' % (pretty_name)
                 context['actions_taken'].append(message)
             except: 
-                log_exception()
+                logger.exception('Place saving %r failed' % pretty_name)
                 message = 'Error adding place "%s"' % pretty_name
                 context['errors'].append(message)
                 continue
@@ -323,7 +322,7 @@ class PlaceAdmin(OSMModelAdmin):
 
     def _show_import_csv_form(self, request, import_form):
         opts = self.model._meta
-        adminform = helpers.AdminForm(import_form, [(None, {'fields': import_form.base_fields.keys()})], {})
+        #adminform = helpers.AdminForm(import_form, [(None, {'fields': import_form.base_fields.keys()})], {})
         context = {
             'title': 'Import %s' % force_unicode(opts.verbose_name_plural),
         }
@@ -367,15 +366,18 @@ class PlaceAdmin(OSMModelAdmin):
 
 class BlockAdmin(OSMModelAdmin):
 
-    list_display = ('pretty_name', 'street', 'suffix', 'left_zip', 'right_zip', 'left_city', 'right_city')
-    list_filter = ('suffix', 'left_city', 'right_city', 'left_zip', 'right_zip')
+    list_display = ('pretty_name', 'predir', 'prefix', 'street', 'suffix', 'postdir',
+                    'left_zip', 'right_zip', 'left_city', 'right_city')
+    list_filter = ('suffix', 'prefix', 'left_city', 'right_city', 'left_zip', 'right_zip',
+                   'predir', 'postdir',
+                   )
     search_fields = ('pretty_name',)
     readonly_fields = ('from_num', 'to_num',)
 
     fieldsets = (
         ('Name', {
                 'fields': ('street_slug', 'pretty_name', 'street_pretty_name',
-                           'predir', 'street', 'suffix', 'postdir',
+                           'predir', 'prefix', 'street', 'suffix', 'postdir',
                            )
                 }),
         ('Address Ranges', {
@@ -401,7 +403,7 @@ class PlaceTypeAdmin(OSMModelAdmin):
 
 class StreetAdmin(OSMModelAdmin):
     list_display = ('pretty_name', 'suffix', 'city', 'state',)
-    list_filter = ('suffix', 'city', 'state',)
+    list_filter = ('prefix', 'suffix', 'city', 'state',)
     search_fields = ('pretty_name',)
     readonly_fields = ('street',)
     prepopulated_fields = {'street_slug': ('pretty_name',)}
