@@ -1,3 +1,21 @@
+#   Copyright 2011 OpenPlans and contributors
+#
+#   This file is part of ebpub
+#
+#   ebpub is free software: you can redistribute it and/or modify
+#   it under the terms of the GNU General Public License as published by
+#   the Free Software Foundation, either version 3 of the License, or
+#   (at your option) any later version.
+#
+#   ebpub is distributed in the hope that it will be useful,
+#   but WITHOUT ANY WARRANTY; without even the implied warranty of
+#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+#   GNU General Public License for more details.
+#
+#   You should have received a copy of the GNU General Public License
+#   along with ebpub.  If not, see <http://www.gnu.org/licenses/>.
+#
+
 from django.contrib.gis import geos
 from django.test import TestCase
 from ebpub.streets.models import Block, Place, PlaceSynonym, PlaceType
@@ -23,58 +41,58 @@ class TestPlaces(TestCase):
         test importing a csv full of places with no errors
         """
         
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0\nDonut House,124 Fakey St.,1.001,2.001")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
         
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
         
     def test_import_csv_synonyms(self):
         """
         test importing synonyms
         """
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,,Big Dough, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,,House of D, D House")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
         
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGH' in synonyms
         assert 'DOUGH MO' in synonyms
 
                 
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
@@ -83,34 +101,34 @@ class TestPlaces(TestCase):
         test changing synonyms via import
         """
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,,Big Dough, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,,House of D, D House")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
         
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGH' in synonyms
         assert 'DOUGH MO' in synonyms
 
                 
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
@@ -118,28 +136,28 @@ class TestPlaces(TestCase):
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,,Ole Doughy, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,,Dunky H, D House")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
         
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'OLE DOUGHY' in synonyms
         assert 'DOUGH MO' in synonyms
 
                 
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'DUNKY H' in synonyms
         assert 'D HOUSE' in synonyms
         
@@ -149,25 +167,25 @@ class TestPlaces(TestCase):
         """
         tests locationless places are not imported.
         """
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0\nFlapper Jacks,,,,\nDonut House,124 Fakey St.,1.001,2.001\nSoup Sacks,,,")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
                 
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
     @mock.patch('ebpub.streets.models.get_metro')
     def test_import_csv_geocoding(self, mock_get_metro):
@@ -178,21 +196,21 @@ class TestPlaces(TestCase):
         mock_get_metro.return_value = {'city_name': 'CHICAGO',
                                        'multiple_cities': False}
 
-        assert Place.objects.all().count() == 0
-        assert test_client_login(self.client, username='admin@example.org', password='123') == True
+        self.assertEqual(Place.objects.all().count(), 0)
+        self.assertEqual(True,
+                         test_client_login(self.client, username='admin@example.org', password='123'))
 
         csv_file = StringIO("Donut Arms Hotel, 205 South Wabash St.,,")
         csv_file.name = 'test.csv'
 
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-        
-        assert Place.objects.all().count() == 1
-        
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Place.objects.all().count(), 1)
+
         place = Place.objects.get(normalized_name='DONUT ARMS HOTEL')
-        assert place.address == '205 South Wabash St.'
-        assert place.location.x - -87.626153836734701 < 0.1
-        assert place.location.y - 41.879488336734696 < 0.1
+        self.assertEqual(place.address, '205 South Wabash St.')
+        self.assertAlmostEqual(place.location.x, -87.626153836734701, places=4)
+        self.assertAlmostEqual(place.location.y, 41.879488336734696, places=4)
 
     @mock.patch('ebpub.streets.models.get_metro')    
     def test_import_csv_lat_lon_priority(self, mock_get_metro):
@@ -203,42 +221,42 @@ class TestPlaces(TestCase):
         mock_get_metro.return_value = {'city_name': 'CHICAGO',
                                        'multiple_cities': False}
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Arms Hotel, 205 South Wabash St.,1.0,2.0")
         csv_file.name = 'test.csv'
 
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 1
+        self.assertEqual(Place.objects.all().count(), 1)
         
         place = Place.objects.get(normalized_name='DONUT ARMS HOTEL')
-        assert place.address == '205 South Wabash St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '205 South Wabash St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
 
 
     def test_import_csv_changes_place_types(self):
         """
         test importing a place that changes a placetype
         """
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 1
+        self.assertEqual(Place.objects.all().count(), 1)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-        assert place.place_type.id == 1
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+        self.assertEqual(place.place_type.id, 1)
 
         new_type = PlaceType(name='ZZZZ', plural_name='ZZZZs', 
                              indefinite_article='a', slug='funhouse',
@@ -248,15 +266,15 @@ class TestPlaces(TestCase):
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '%d' % new_type.id, 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
-        assert Place.objects.all().count() == 1
+        self.assertEqual(Place.objects.all().count(), 1)
         
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-        assert place.place_type.id == new_type.id
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+        self.assertEqual(place.place_type.id, new_type.id)
 
     def test_export_csv(self):
         """
@@ -281,23 +299,23 @@ class TestPlaces(TestCase):
         
         count = 0
         for row in rows:
-            assert len(row) == 5
+            self.assertEqual(len(row), 5)
             if row[0] == 'Donut Palace':
-                assert row[1] == '100 Bubble Street'
-                assert row[2] == '2.0'
-                assert row[3] == '1.0'
-                assert row[4] == ''       
+                self.assertEqual(row[1], '100 Bubble Street')
+                self.assertEqual(row[2], '2.0')
+                self.assertEqual(row[3], '1.0')
+                self.assertEqual(row[4], '')
             elif row[0] == 'Donut Sanctuary':
-                assert row[1] == '101 Bubble Street'
-                assert row[2] == '4.0'
-                assert row[3] == '3.0'
-                assert row[4] == 'http://www.example.org/bs'
+                self.assertEqual(row[1], '101 Bubble Street')
+                self.assertEqual(row[2], '4.0')
+                self.assertEqual(row[3], '3.0')
+                self.assertEqual(row[4], 'http://www.example.org/bs')
             else: 
-                assert 0, 'Unexpected Place!'       
+                self.fail('Unexpected Place! %s' % row[0])
             count += 1
-            
-        assert count == 2
-        
+
+        self.assertEqual(count, 2)
+
 
     def test_export_csv_synonyms(self):
         """
@@ -325,85 +343,83 @@ class TestPlaces(TestCase):
         ps.save()
         
         response = self.client.post(self.export_url, {'place_type': place_type.id})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
         
         rows = csv.reader(StringIO(response.content))
         
         count = 0
         for row in rows:
-            assert len(row) == 7
+            self.assertEqual(len(row), 7)
             synonyms = set(row[5:])
             if row[0] == 'Donut Palace':
-                assert row[1] == '100 Bubble Street'
-                assert row[2] == '2.0'
-                assert row[3] == '1.0'
-                assert row[4] == ''
+                self.assertEqual(row[1], '100 Bubble Street')
+                self.assertEqual(row[2], '2.0')
+                self.assertEqual(row[3], '1.0')
+                self.assertEqual(row[4], '')
                 assert 'Donut Hole' in synonyms
                 assert 'Donut Pally' in synonyms
             elif row[0] == 'Donut Sanctuary':
-                assert row[1] == '101 Bubble Street'
-                assert row[2] == '4.0'
-                assert row[3] == '3.0'
-                assert row[4] == 'http://www.example.org/bs'
+                self.assertEqual(row[1], '101 Bubble Street')
+                self.assertEqual(row[2], '4.0')
+                self.assertEqual(row[3], '3.0')
+                self.assertEqual(row[4], 'http://www.example.org/bs')
                 assert 'Sancy D' in synonyms
                 assert 'D Sanc' in synonyms
             else: 
-                assert 0, 'Unexpected Place!'       
+                self.fail('Unexpected Place!' % row[0])
             count += 1
-            
-        assert count == 2
-        
-        
+
+        self.assertEqual(count, 2)
+
+
     def test_import_export(self):
         """
         tests that the results of an export can be imported
         """
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0\nFlapper Jacks,,,,\nDonut House,124 Fakey St.,1.001,2.001,http://www.example.org/bs\nSoup Sacks,,,")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-        
-        assert Place.objects.all().count() == 2
-        
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Place.objects.all().count(), 2)
+
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-                
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
-        assert place.url == 'http://www.example.org/bs'
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
+        self.assertEqual(place.url, 'http://www.example.org/bs')
 
         response = self.client.post(self.export_url, {'place_type': place.place_type.id})
-        assert response.status_code == 200
-        
+        self.assertEqual(response.status_code, 200)
+
         Place.objects.all().delete()
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
 
         csv_file = StringIO(response.content)
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-        
-        assert Place.objects.all().count() == 2
-        
+        self.assertEqual(response.status_code, 200)
+
+        self.assertEqual(Place.objects.all().count(), 2)
+
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-                
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
-        assert place.url == 'http://www.example.org/bs'
-        
-        
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
+        self.assertEqual(place.url, 'http://www.example.org/bs')
+
 
     def test_import_export_synonyms(self):
         """
@@ -411,70 +427,67 @@ class TestPlaces(TestCase):
         including synonyms
         """
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,,Big Dough, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,http://www.example.org/bs,House of D, D House")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-        
-        assert Place.objects.all().count() == 2
-        
+        self.assertEqual(response.status_code, 200)
+
+        self.assertEqual(Place.objects.all().count(), 2)
+
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-        
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGH' in synonyms
         assert 'DOUGH MO' in synonyms
-        
-                
+
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
-        
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
+
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
-
         response = self.client.post(self.export_url, {'place_type': place.place_type.id})
-        assert response.status_code == 200
-        
+        self.assertEqual(response.status_code, 200)
+
         Place.objects.all().delete()
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
 
         csv_file = StringIO(response.content)
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-        
-        assert Place.objects.all().count() == 2
-        
+        self.assertEqual(response.status_code, 200)
+
+        self.assertEqual(Place.objects.all().count(), 2)
+
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
-        
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
+
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGH' in synonyms
         assert 'DOUGH MO' in synonyms
-        
-                
+
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
-        
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
+
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
@@ -484,15 +497,14 @@ class TestPlaces(TestCase):
         tests that the importer can handle places with the same name.
         """
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,http://www.example.org/bs/0\nDonut Mountain,99 Fakley St.,99.0,22.0,http://www.example.org/bs/1")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-
-        assert Place.objects.all().count() == 2
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Place.objects.all().count(), 2)
 
         locs = []
         for place in Place.objects.filter(normalized_name='DONUT MOUNTAIN').all():
@@ -500,8 +512,7 @@ class TestPlaces(TestCase):
 
         assert ('123 Fakey St.', 2.0, 1.0, 'http://www.example.org/bs/0') in locs
         assert ('99 Fakley St.', 22.0, 99.0, 'http://www.example.org/bs/1') in locs
-        
-        
+
 
     def test_import_same_name_synonym(self):
         """
@@ -509,15 +520,15 @@ class TestPlaces(TestCase):
         and maintain separate synonym lists.
         """
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,http://www.example.org/bs/0,123 D House, Mount D\nDonut Mountain,99 Fakley St.,99.0,22.0,http://www.example.org/bs/1,99 D House, Mount D")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
 
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
 
         locs = []
         for place in Place.objects.filter(normalized_name='DONUT MOUNTAIN').all():
@@ -537,20 +548,20 @@ class TestPlaces(TestCase):
         to modify things, and no duplicates are formed.
         """
 
-        assert Place.objects.all().count() == 0
+        self.assertEqual(Place.objects.all().count(), 0)
         assert test_client_login(self.client, username='admin@example.org', password='123') == True
 
         csv_file = StringIO("Donut Mountain,123 Fakey St.,1.0,2.0,,Big Dough, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,http://www.example.org/bs,House of D, D House")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
+        self.assertEqual(response.status_code, 200)
 
-        assert Place.objects.all().count() == 2
+        self.assertEqual(Place.objects.all().count(), 2)
 
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
         assert len(synonyms) == 2
@@ -559,39 +570,37 @@ class TestPlaces(TestCase):
 
 
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
         # re-import
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-
-        assert Place.objects.all().count() == 2
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Place.objects.all().count(), 2)
 
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '123 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '123 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGH' in synonyms
         assert 'DOUGH MO' in synonyms
 
-
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' in synonyms
 
@@ -599,32 +608,29 @@ class TestPlaces(TestCase):
         csv_file = StringIO("Donut Mountain,121 Fakey St.,1.0,2.0,,Big Doughy, Dough Mo\nDonut House,124 Fakey St.,1.001,2.001,http://www.example.org/bs, D House,Meye Donuts")
         csv_file.name = 'test.csv'
         response = self.client.post(self.import_url, {'place_type': '1', 'csv_file': csv_file})
-        assert response.status_code == 200
-
-        assert Place.objects.all().count() == 2
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(Place.objects.all().count(), 2)
 
         place = Place.objects.get(normalized_name='DONUT MOUNTAIN')
-        assert place.address == '121 Fakey St.'
-        assert place.location.x == 2.0
-        assert place.location.y == 1.0
+        self.assertEqual(place.address, '121 Fakey St.')
+        self.assertEqual(place.location.x, 2.0)
+        self.assertEqual(place.location.y, 1.0)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'BIG DOUGHY' in synonyms
         assert 'DOUGH MO' in synonyms
 
-
         place = Place.objects.get(normalized_name='DONUT HOUSE')
-        assert place.address == '124 Fakey St.'
-        assert place.location.x == 2.001
-        assert place.location.y == 1.001
+        self.assertEqual(place.address, '124 Fakey St.')
+        self.assertEqual(place.location.x, 2.001)
+        self.assertEqual(place.location.y, 1.001)
 
         synonyms = set([x.normalized_name for x in PlaceSynonym.objects.filter(place=place).all()])
-        assert len(synonyms) == 2
+        self.assertEqual(len(synonyms), 2)
         assert 'D HOUSE' in synonyms
         assert 'HOUSE OF D' not in synonyms
         assert 'MEYE DONUTS' in synonyms
-        
 
 
 class TestBlocks(TestCase):
